@@ -3,7 +3,7 @@
  *  BlueZ - Bluetooth protocol stack for Linux
  *
  *  Copyright (C) 2006-2007  Nokia Corporation
- *  Copyright (C) 2004-2008  Marcel Holtmann <marcel@holtmann.org>
+ *  Copyright (C) 2004-2009  Marcel Holtmann <marcel@holtmann.org>
  *  Copyright (C) 2005-2007  Johan Hedberg <johan.hedberg@nokia.com>
  *
  *
@@ -32,7 +32,6 @@
 #include <stdlib.h>
 #include <string.h>
 #include <unistd.h>
-#include <fcntl.h>
 #include <sys/ioctl.h>
 
 #include <bluetooth/bluetooth.h>
@@ -54,57 +53,6 @@
 
 #define RECONNECT_RETRY_TIMEOUT	5000
 
-int l2raw_connect(const bdaddr_t *src, const bdaddr_t *dst)
-{
-	struct sockaddr_l2 addr;
-	long arg;
-	int sk;
-
-	sk = socket(PF_BLUETOOTH, SOCK_RAW, BTPROTO_L2CAP);
-	if (sk < 0) {
-		error("Can't create socket: %s (%d)", strerror(errno), errno);
-		return sk;
-	}
-
-	memset(&addr, 0, sizeof(addr));
-	addr.l2_family = AF_BLUETOOTH;
-	bacpy(&addr.l2_bdaddr, src);
-
-	if (bind(sk, (struct sockaddr *) &addr, sizeof(addr)) < 0) {
-		error("Can't bind socket: %s (%d)", strerror(errno), errno);
-		goto failed;
-	}
-
-	arg = fcntl(sk, F_GETFL);
-	if (arg < 0) {
-		error("Can't get file flags: %s (%d)", strerror(errno), errno);
-		goto failed;
-	}
-
-	arg |= O_NONBLOCK;
-	if (fcntl(sk, F_SETFL, arg) < 0) {
-		error("Can't set file flags: %s (%d)", strerror(errno), errno);
-		goto failed;
-	}
-
-	memset(&addr, 0, sizeof(addr));
-	addr.l2_family = AF_BLUETOOTH;
-	bacpy(&addr.l2_bdaddr, dst);
-
-	if (connect(sk, (struct sockaddr *) &addr, sizeof(addr)) < 0) {
-		if (errno == EAGAIN || errno == EINPROGRESS)
-			return sk;
-		error("Can't connect socket: %s (%d)", strerror(errno), errno);
-		goto failed;
-	}
-
-	return sk;
-
-failed:
-	close(sk);
-	return -1;
-}
-
 static gboolean system_bus_reconnect(void *data)
 {
 	DBusConnection *conn = get_dbus_connection();
@@ -217,19 +165,27 @@ void hcid_dbus_exit(void)
 int hcid_dbus_init(void)
 {
 	DBusConnection *conn;
+	DBusError err;
 
-	conn = g_dbus_setup_bus(DBUS_BUS_SYSTEM, BLUEZ_NAME, NULL);
-	if (!conn)
-		return -1;
+	dbus_error_init(&err);
+
+	conn = g_dbus_setup_bus(DBUS_BUS_SYSTEM, BLUEZ_NAME, &err);
+	if (!conn) {
+		if (dbus_error_is_set(&err)) {
+			dbus_error_free(&err);
+			return -EIO;
+		}
+		return -EALREADY;
+	}
 
 	if (g_dbus_set_disconnect_function(conn, disconnect_callback,
 							NULL, NULL) == FALSE) {
 		dbus_connection_unref(conn);
-		return -1;
+		return -EIO;
 	}
 
 	if (!manager_init(conn, "/"))
-		return -1;
+		return -EIO;
 
 	set_dbus_connection(conn);
 
